@@ -14,13 +14,11 @@ from collections.abc import Iterable, Iterator, Mapping, Sequence
 from numbers import Real
 from types import SimpleNamespace
 from typing import (
-    TYPE_CHECKING,
     Any,
     Literal,
     NamedTuple,
-    Optional,
+    TypeAlias,
     TypeVar,
-    Union,
     get_args,
 )
 from warnings import warn
@@ -55,18 +53,15 @@ from hypothesis.internal.validation import (
 from hypothesis.strategies._internal.strategies import check_strategy
 from hypothesis.strategies._internal.utils import defines_strategy
 
-if TYPE_CHECKING:
-    from typing import TypeAlias
-
 __all__ = [
     "make_strategies_namespace",
 ]
 
 
-RELEASED_VERSIONS = ("2021.12", "2022.12", "2023.12")
+RELEASED_VERSIONS = ("2021.12", "2022.12", "2023.12", "2024.12")
 NOMINAL_VERSIONS = (*RELEASED_VERSIONS, "draft")
 assert sorted(NOMINAL_VERSIONS) == list(NOMINAL_VERSIONS)  # sanity check
-NominalVersion = Literal["2021.12", "2022.12", "2023.12", "draft"]
+NominalVersion = Literal["2021.12", "2022.12", "2023.12", "2024.12", "draft"]
 assert get_args(NominalVersion) == NOMINAL_VERSIONS  # sanity check
 
 
@@ -118,7 +113,7 @@ def warn_on_missing_dtypes(xp: Any, stubs: list[str]) -> None:
 
 def find_castable_builtin_for_dtype(
     xp: Any, api_version: NominalVersion, dtype: DataType
-) -> type[Union[bool, int, float, complex]]:
+) -> type[bool | int | float | complex]:
     """Returns builtin type which can have values that are castable to the given
     dtype, according to :xp-ref:`type promotion rules <type_promotion.html>`.
 
@@ -180,16 +175,16 @@ def dtype_from_name(xp: Any, name: str) -> Any:
 def _from_dtype(
     xp: Any,
     api_version: NominalVersion,
-    dtype: Union[DataType, str],
+    dtype: DataType | str,
     *,
-    min_value: Optional[Union[int, float]] = None,
-    max_value: Optional[Union[int, float]] = None,
-    allow_nan: Optional[bool] = None,
-    allow_infinity: Optional[bool] = None,
-    allow_subnormal: Optional[bool] = None,
-    exclude_min: Optional[bool] = None,
-    exclude_max: Optional[bool] = None,
-) -> st.SearchStrategy[Union[bool, int, float, complex]]:
+    min_value: int | float | None = None,
+    max_value: int | float | None = None,
+    allow_nan: bool | None = None,
+    allow_infinity: bool | None = None,
+    allow_subnormal: bool | None = None,
+    exclude_min: bool | None = None,
+    exclude_max: bool | None = None,
+) -> st.SearchStrategy[bool | int | float | complex]:
     """Return a strategy for any value of the given dtype.
 
     Values generated are of the Python scalar which is
@@ -313,6 +308,7 @@ class ArrayStrategy(st.SearchStrategy):
     def __init__(
         self, *, xp, api_version, elements_strategy, dtype, shape, fill, unique
     ):
+        super().__init__()
         self.xp = xp
         self.elements_strategy = elements_strategy
         self.dtype = dtype
@@ -425,8 +421,8 @@ class ArrayStrategy(st.SearchStrategy):
                     if val in seen:
                         elements.reject("chose an element we've already used")
                         continue
-                    else:
-                        seen.add(val)
+                    seen.add(val)
+
                 result_obj[i] = val
                 assigned.add(i)
                 fill_mask[i] = False
@@ -455,11 +451,11 @@ class ArrayStrategy(st.SearchStrategy):
 def _arrays(
     xp: Any,
     api_version: NominalVersion,
-    dtype: Union[DataType, str, st.SearchStrategy[DataType], st.SearchStrategy[str]],
-    shape: Union[int, Shape, st.SearchStrategy[Shape]],
+    dtype: DataType | str | st.SearchStrategy[DataType] | st.SearchStrategy[str],
+    shape: int | Shape | st.SearchStrategy[Shape],
     *,
-    elements: Optional[Union[Mapping[str, Any], st.SearchStrategy]] = None,
-    fill: Optional[st.SearchStrategy[Any]] = None,
+    elements: Mapping[str, Any] | st.SearchStrategy | None = None,
+    fill: st.SearchStrategy[Any] | None = None,
     unique: bool = False,
 ) -> st.SearchStrategy:
     """Returns a strategy for :xp-ref:`arrays <array_object.html>`.
@@ -499,9 +495,6 @@ def _arrays(
       >>> xps.arrays(xp, xp.int8, 3, elements={"min_value": 10}).example()
       Array([125, 13, 79], dtype=int8)
 
-    Refer to :doc:`What you can generate and how <data>` for passing
-    your own elements strategy.
-
     .. code-block:: pycon
 
       >>> xps.arrays(xp, xp.float32, 3, elements=floats(0, 1, width=32)).example()
@@ -652,13 +645,13 @@ def numeric_dtype_names(base_name: str, sizes: Sequence[int]) -> Iterator[str]:
         yield f"{base_name}{size}"
 
 
-IntSize: "TypeAlias" = Literal[8, 16, 32, 64]
-FltSize: "TypeAlias" = Literal[32, 64]
-CpxSize: "TypeAlias" = Literal[64, 128]
+IntSize: TypeAlias = Literal[8, 16, 32, 64]
+FltSize: TypeAlias = Literal[32, 64]
+CpxSize: TypeAlias = Literal[64, 128]
 
 
 def _integer_dtypes(
-    xp: Any, *, sizes: Union[IntSize, Sequence[IntSize]] = (8, 16, 32, 64)
+    xp: Any, *, sizes: IntSize | Sequence[IntSize] = (8, 16, 32, 64)
 ) -> st.SearchStrategy[DataType]:
     """Return a strategy for signed integer dtype objects.
 
@@ -676,7 +669,7 @@ def _integer_dtypes(
 
 
 def _unsigned_integer_dtypes(
-    xp: Any, *, sizes: Union[IntSize, Sequence[IntSize]] = (8, 16, 32, 64)
+    xp: Any, *, sizes: IntSize | Sequence[IntSize] = (8, 16, 32, 64)
 ) -> st.SearchStrategy[DataType]:
     """Return a strategy for unsigned integer dtype objects.
 
@@ -696,7 +689,7 @@ def _unsigned_integer_dtypes(
 
 
 def _floating_dtypes(
-    xp: Any, *, sizes: Union[FltSize, Sequence[FltSize]] = (32, 64)
+    xp: Any, *, sizes: FltSize | Sequence[FltSize] = (32, 64)
 ) -> st.SearchStrategy[DataType]:
     """Return a strategy for real-valued floating-point dtype objects.
 
@@ -714,7 +707,7 @@ def _floating_dtypes(
 
 
 def _complex_dtypes(
-    xp: Any, *, sizes: Union[CpxSize, Sequence[CpxSize]] = (64, 128)
+    xp: Any, *, sizes: CpxSize | Sequence[CpxSize] = (64, 128)
 ) -> st.SearchStrategy[DataType]:
     """Return a strategy for complex dtype objects.
 
@@ -751,9 +744,9 @@ def mutually_broadcastable_shapes(
     *,
     base_shape: Shape = (),
     min_dims: int = 0,
-    max_dims: Optional[int] = None,
+    max_dims: int | None = None,
     min_side: int = 1,
-    max_side: Optional[int] = None,
+    max_side: int | None = None,
 ) -> st.SearchStrategy[BroadcastableShapes]:
     return _mutually_broadcastable_shapes(
         num_shapes=num_shapes,
@@ -773,7 +766,7 @@ def indices(
     shape: Shape,
     *,
     min_dims: int = 0,
-    max_dims: Optional[int] = None,
+    max_dims: int | None = None,
     allow_newaxis: bool = False,
     allow_ellipsis: bool = True,
 ) -> st.SearchStrategy[BasicIndex]:
@@ -847,7 +840,7 @@ _args_to_xps: WeakValueDictionary = WeakValueDictionary()
 
 
 def make_strategies_namespace(
-    xp: Any, *, api_version: Optional[NominalVersion] = None
+    xp: Any, *, api_version: NominalVersion | None = None
 ) -> SimpleNamespace:
     """Creates a strategies namespace for the given array module.
 
@@ -925,16 +918,16 @@ def make_strategies_namespace(
 
     @defines_strategy(force_reusable_values=True)
     def from_dtype(
-        dtype: Union[DataType, str],
+        dtype: DataType | str,
         *,
-        min_value: Optional[Union[int, float]] = None,
-        max_value: Optional[Union[int, float]] = None,
-        allow_nan: Optional[bool] = None,
-        allow_infinity: Optional[bool] = None,
-        allow_subnormal: Optional[bool] = None,
-        exclude_min: Optional[bool] = None,
-        exclude_max: Optional[bool] = None,
-    ) -> st.SearchStrategy[Union[bool, int, float, complex]]:
+        min_value: int | float | None = None,
+        max_value: int | float | None = None,
+        allow_nan: bool | None = None,
+        allow_infinity: bool | None = None,
+        allow_subnormal: bool | None = None,
+        exclude_min: bool | None = None,
+        exclude_max: bool | None = None,
+    ) -> st.SearchStrategy[bool | int | float | complex]:
         return _from_dtype(
             xp,
             api_version,
@@ -950,13 +943,11 @@ def make_strategies_namespace(
 
     @defines_strategy(force_reusable_values=True)
     def arrays(
-        dtype: Union[
-            DataType, str, st.SearchStrategy[DataType], st.SearchStrategy[str]
-        ],
-        shape: Union[int, Shape, st.SearchStrategy[Shape]],
+        dtype: DataType | str | st.SearchStrategy[DataType] | st.SearchStrategy[str],
+        shape: int | Shape | st.SearchStrategy[Shape],
         *,
-        elements: Optional[Union[Mapping[str, Any], st.SearchStrategy]] = None,
-        fill: Optional[st.SearchStrategy[Any]] = None,
+        elements: Mapping[str, Any] | st.SearchStrategy | None = None,
+        fill: st.SearchStrategy[Any] | None = None,
         unique: bool = False,
     ) -> st.SearchStrategy:
         return _arrays(
@@ -987,19 +978,19 @@ def make_strategies_namespace(
 
     @defines_strategy()
     def integer_dtypes(
-        *, sizes: Union[IntSize, Sequence[IntSize]] = (8, 16, 32, 64)
+        *, sizes: IntSize | Sequence[IntSize] = (8, 16, 32, 64)
     ) -> st.SearchStrategy[DataType]:
         return _integer_dtypes(xp, sizes=sizes)
 
     @defines_strategy()
     def unsigned_integer_dtypes(
-        *, sizes: Union[IntSize, Sequence[IntSize]] = (8, 16, 32, 64)
+        *, sizes: IntSize | Sequence[IntSize] = (8, 16, 32, 64)
     ) -> st.SearchStrategy[DataType]:
         return _unsigned_integer_dtypes(xp, sizes=sizes)
 
     @defines_strategy()
     def floating_dtypes(
-        *, sizes: Union[FltSize, Sequence[FltSize]] = (32, 64)
+        *, sizes: FltSize | Sequence[FltSize] = (32, 64)
     ) -> st.SearchStrategy[DataType]:
         return _floating_dtypes(xp, sizes=sizes)
 
@@ -1037,30 +1028,30 @@ def make_strategies_namespace(
                 f_args += f", api_version='{self.api_version}'"
             return f"make_strategies_namespace({f_args})"
 
-    kwargs = dict(
-        name=xp.__name__,
-        api_version=api_version,
-        from_dtype=from_dtype,
-        arrays=arrays,
-        array_shapes=array_shapes,
-        scalar_dtypes=scalar_dtypes,
-        boolean_dtypes=boolean_dtypes,
-        real_dtypes=real_dtypes,
-        numeric_dtypes=numeric_dtypes,
-        integer_dtypes=integer_dtypes,
-        unsigned_integer_dtypes=unsigned_integer_dtypes,
-        floating_dtypes=floating_dtypes,
-        valid_tuple_axes=valid_tuple_axes,
-        broadcastable_shapes=broadcastable_shapes,
-        mutually_broadcastable_shapes=mutually_broadcastable_shapes,
-        indices=indices,
-    )
+    kwargs = {
+        "name": xp.__name__,
+        "api_version": api_version,
+        "from_dtype": from_dtype,
+        "arrays": arrays,
+        "array_shapes": array_shapes,
+        "scalar_dtypes": scalar_dtypes,
+        "boolean_dtypes": boolean_dtypes,
+        "real_dtypes": real_dtypes,
+        "numeric_dtypes": numeric_dtypes,
+        "integer_dtypes": integer_dtypes,
+        "unsigned_integer_dtypes": unsigned_integer_dtypes,
+        "floating_dtypes": floating_dtypes,
+        "valid_tuple_axes": valid_tuple_axes,
+        "broadcastable_shapes": broadcastable_shapes,
+        "mutually_broadcastable_shapes": mutually_broadcastable_shapes,
+        "indices": indices,
+    }
 
     if api_version > "2021.12":
 
         @defines_strategy()
         def complex_dtypes(
-            *, sizes: Union[CpxSize, Sequence[CpxSize]] = (64, 128)
+            *, sizes: CpxSize | Sequence[CpxSize] = (64, 128)
         ) -> st.SearchStrategy[DataType]:
             return _complex_dtypes(xp, sizes=sizes)
 
