@@ -13,9 +13,10 @@ import gc
 import random
 import sys
 import warnings
-from collections.abc import Generator, Hashable
+from collections.abc import Callable, Generator, Hashable
 from itertools import count
-from typing import TYPE_CHECKING, Any, Callable, Optional
+from random import Random
+from typing import TYPE_CHECKING, Any
 from weakref import WeakValueDictionary
 
 import hypothesis.core
@@ -57,7 +58,7 @@ class NumpyRandomWrapper:
         self.setstate = numpy.random.set_state
 
 
-NP_RANDOM: Optional[RandomLike] = None
+NP_RANDOM: RandomLike | None = None
 
 
 if not (PYPY or GRAALPY):
@@ -116,7 +117,11 @@ def register_random(r: RandomLike) -> None:
     if not (hasattr(r, "seed") and hasattr(r, "getstate") and hasattr(r, "setstate")):
         raise InvalidArgument(f"{r=} does not have all the required methods")
 
-    if r in RANDOMS_TO_MANAGE.values():
+    if r in [
+        random
+        for ref in RANDOMS_TO_MANAGE.data.copy().values()  # type: ignore
+        if (random := ref()) is not None
+    ]:
         return
 
     if not (PYPY or GRAALPY):  # pragma: no branch
@@ -149,15 +154,12 @@ def register_random(r: RandomLike) -> None:
     RANDOMS_TO_MANAGE[next(_RKEY)] = r
 
 
-# the most recent state of the global random instance, as set by hypothesis.
-# This might not be the current state of the global random instance if its
-# state changed since hypothesis seeded it. If nobody other than hypothesis
-# is touching the global random instance, then this will be the state of the global
-# random instance.
-#
-# This is used to address a threading race condition in deprecate_random_in_strategy.
-_most_recent_random_state_enter: Optional[Any] = None
-_most_recent_random_state_exit: Optional[Any] = None
+# Used to make the warning issued by `deprecate_random_in_strategy` thread-safe,
+# as well as to avoid warning on uses of st.randoms().
+# Store just the hash to reduce memory consumption. This is an underapproximation
+# of membership (distinct items might have the same hash), which is fine for the
+# warning, as it results in missed alarms, not false alarms.
+_known_random_state_hashes: set[Any] = set()
 
 
 def get_seeder_and_restorer(
@@ -183,32 +185,54 @@ def get_seeder_and_restorer(
             NP_RANDOM = RANDOMS_TO_MANAGE[next(_RKEY)] = NumpyRandomWrapper()
 
     def seed_all() -> None:
-        global _most_recent_random_state_enter
         assert not states
-        for k, r in RANDOMS_TO_MANAGE.items():
+        # access .data.copy().items() instead of .items() to avoid a "dictionary
+        # changed size during iteration" error under multithreading.
+        #
+        # I initially expected this to be fixed by
+        # https://github.com/python/cpython/commit/96d37dbcd23e65a7a57819aeced9034296ef747e,
+        # but I believe that is addressing the size change from weakrefs expiring
+        # during gc, not from the user adding new elements to the dict.
+        #
+        # Since we're accessing .data, we have to manually handle checking for
+        # expired ref instances during iteration. Normally WeakValueDictionary
+        # handles this for us.
+        #
+        # This command reproduces at time of writing:
+        #   pytest hypothesis-python/tests/ -k test_intervals_are_equivalent_to_their_lists
+        #   --parallel-threads 2
+        for k, ref in RANDOMS_TO_MANAGE.data.copy().items():  # type: ignore
+            r = ref()
+            if r is None:
+                # ie the random instance has been gc'd
+                continue  # pragma: no cover
             states[k] = r.getstate()
-            r.seed(seed)
             if k == _global_random_rkey:
-                # setting a seed is equivalent to setting setstate, so we need
-                # to track it globally for race conditions.
+                # r.seed sets the random's state. We want to add that state to
+                # _known_random_states before calling r.seed, in case a thread
+                # switch occurs between the two. To figure out the seed -> state
+                # mapping, set the seed on a dummy random and add that state to
+                # _known_random_state.
                 #
-                # I think there's still a race here if a thread switch occurs
-                # after r.seed but before we set _most_recent_random_state_enter.
-                # We'd need to seed a dummy Random instance to figure out the
-                # seed -> state mapping, then set _most_recent_random_state_enter,
-                # then call setstate (or equivalently .seed) on the real random.
-                _most_recent_random_state_enter = r.getstate()
+                # we could use a global dummy random here, but then we'd have to
+                # put a lock around it, and it's not clear to me if that's more
+                # efficient than constructing a new instance each time.
+                dummy_random = Random()
+                dummy_random.seed(seed)
+                _known_random_state_hashes.add(hash(dummy_random.getstate()))
+                # we expect `assert r.getstate() == dummy_random.getstate()` to
+                # hold here, but thread switches means it might not.
+
+            r.seed(seed)
 
     def restore_all() -> None:
-        global _most_recent_random_state_exit
-
         for k, state in states.items():
             r = RANDOMS_TO_MANAGE.get(k)
             if r is None:  # i.e., has been garbage-collected
                 continue
 
             if k == _global_random_rkey:
-                _most_recent_random_state_exit = state
+                _known_random_state_hashes.add(hash(state))
             r.setstate(state)
 
         states.clear()
@@ -220,7 +244,7 @@ def get_seeder_and_restorer(
 def deterministic_PRNG(seed: int = 0) -> Generator[None, None, None]:
     """Context manager that handles random.seed without polluting global state.
 
-    See issue #1255 and PR #1295 for details and motivation - in short,
+    See issue #1266 and PR #1295 for details and motivation - in short,
     leaving the global pseudo-random number generator (PRNG) seeded is a very
     bad idea in principle, and breaks all kinds of independence assumptions
     in practice.
@@ -228,7 +252,7 @@ def deterministic_PRNG(seed: int = 0) -> Generator[None, None, None]:
     if (
         hypothesis.core.threadlocal._hypothesis_global_random is None
     ):  # pragma: no cover
-        hypothesis.core.threadlocal._hypothesis_global_random = random.Random()
+        hypothesis.core.threadlocal._hypothesis_global_random = Random()
         register_random(hypothesis.core.threadlocal._hypothesis_global_random)
 
     seed_all, restore_all = get_seeder_and_restorer(seed)
@@ -237,3 +261,7 @@ def deterministic_PRNG(seed: int = 0) -> Generator[None, None, None]:
         yield
     finally:
         restore_all()
+        # TODO it would be nice to clean up _known_random_state_hashes when no
+        # active deterministic_PRNG contexts remain, to free memory (see similar
+        # logic in StackframeLimiter). But it's a bit annoying to get right, and
+        # likely not a big deal.
